@@ -9,7 +9,7 @@
 #' @param failSafeP  p-value used to determine across which time intervals effects become non-significant
 #' @param timeRange vector describing the time range for x-axis as sequence from/to/stepSize (e.g., c(1, 144, 1))
 #' @param useMBESS use 'MBESS' package to calculate statistical power (slower)
-#' @param coresToUse vector describing the time range for x-axis as sequence from/to/stepSize (e.g., c(1, 144, 1))
+#' @param coresToUse if negative, the value is subtracted from available cores, else value = cores to use
 #' @param indVarying Allows continuous time intercepts to vary at the individual level (random effects model, accounts for unobserved heterogeneity)
 #' @param activateRPB set to TRUE to receive push messages with 'CoTiMA' notifications on your phone
 #' @param silentOverwrite overwrite old files without asking
@@ -183,8 +183,15 @@ ctmaPower <- function(
     # all parameter estimates
     tmp0 <- ctmaInitFit$studyFitList[[1]]$resultsSummary$parmatrices; tmp0
     driftRows <- which(rownames(tmp0) == "DRIFT"); driftRows
+    if (length(driftRows) == 0) driftRows <- which(tmp0[,1] =="DRIFT"); driftRows
+
     diffusionRows <- which(rownames(tmp0)  == "DIFFUSIONcov"); diffusionRows
+    if (length(diffusionRows) == 0) diffusionRows <- which(tmp0[,1] =="DIFFUSIONcov"); diffusionRows
+
     T0varRows <- which(rownames(tmp0) =="T0VAR"); T0varRows
+    if (length(T0varRows) == 0) T0varRows <- which(rownames(tmp0) =="T0cov"); T0varRows
+    if (length(T0varRows) == 0) T0varRows <- which(tmp0[,1] =="T0cov"); T0varRows
+
     {
       tmp1 <- lapply(ctmaInitFit$studyFitList, function(extract) c(extract$resultsSummary$parmatrices[driftRows, 3]) ); tmp1
       tmp1 <- matrix(unlist(tmp1), nrow=n.studies, byrow=TRUE); tmp1
@@ -251,7 +258,8 @@ ctmaPower <- function(
     maxTpoints <- max(allTpoints); maxTpoints # replacement
     allDeltas <- ctmaInitFit$statisticsList$allDeltas; allDeltas
     maxDelta <- max(allDeltas); maxDelta
-    if (is.null(timeRange)) usedTimeRange <- seq(0, 1.5*maxDelta, 1) else usedTimeRange <- timeRange
+    #if (is.null(timeRange)) usedTimeRange <- seq(0, 1.5*maxDelta, 1) else usedTimeRange <- timeRange
+    if (is.null(timeRange)) usedTimeRange <- seq(0, 1.5*maxDelta, 1) else usedTimeRange <- seq(timeRange[1], timeRange[2], timeRange[3])
     # augment by all existing time lags
     usedTimeRange <- sort(unique(c(usedTimeRange, allDeltas))); usedTimeRange
     allTpoints <- ctmaInitFit$statisticsList$allTpoints; allTpoints
@@ -304,7 +312,7 @@ ctmaPower <- function(
     targetCols <- which(colnames(dataTmp) == "groups"); targetCols
     dataTmp <- dataTmp[ ,-targetCols]
     dataTmp2 <- suppressMessages(ctWideToLong(dataTmp, Tpoints=maxTpoints, n.manifest=n.latent, n.TIpred = (n.studies-1),
-                             manifestNames=manifestNames))
+                                              manifestNames=manifestNames))
     dataTmp3 <- suppressMessages(ctDeintervalise(dataTmp2))
     dataTmp3[, "time"] <- dataTmp3[, "time"] * CoTiMAStanctArgs$scaleTime
     # eliminate rows where ALL latents are NA
@@ -371,8 +379,8 @@ ctmaPower <- function(
       tmp1 <- which(allInvDrift_Coeff[, "matrix"] == "DRIFT")
       driftNamesTmp <- c(matrix(driftNames, n.latent, n.latent, byrow=TRUE)); driftNamesTmp
       rownames(allInvDrift_Coeff) <- paste0(allInvDrift_Coeff[, c("matrix")], "_",
-                                              allInvDrift_Coeff[, c("row")], "_",
-                                              allInvDrift_Coeff[, c("col")])
+                                            allInvDrift_Coeff[, c("row")], "_",
+                                            allInvDrift_Coeff[, c("col")])
     } else {
       tmp1 <- which(rownames(allInvDrift_Coeff) == "DRIFT")
       driftNamesTmp <- c(matrix(driftNames, n.latent, n.latent, byrow=FALSE)); driftNamesTmp
@@ -931,7 +939,6 @@ ctmaPower <- function(
   }
 
   allInvModelFit$resultsSummary <- allInvModelFitSummary
-  allInvModelFit$resultsSummary
 
   results <- list(activeDirectory=activeDirectory,
                   plot.type=c("power"), model.type="stanct", #model.type="mx",
